@@ -4,7 +4,7 @@
  * 		Connection management functions for mysql_fdw
  *
  * Portions Copyright (c) 2012-2014, PostgreSQL Global Development Group
- * Portions Copyright (c) 2004-2021, EnterpriseDB Corporation.
+ * Portions Copyright (c) 2004-2022, EnterpriseDB Corporation.
  *
  * IDENTIFICATION
  * 		connection.c
@@ -17,7 +17,6 @@
 #if PG_VERSION_NUM >= 130000
 #include "common/hashfn.h"
 #endif
-#include "mb/pg_wchar.h"
 #include "mysql_fdw.h"
 #include "utils/hsearch.h"
 #include "utils/inval.h"
@@ -120,10 +119,6 @@ mysql_get_connection(ForeignServer *server, UserMapping *user, mysql_opt *opt)
 
 	if (entry->conn == NULL)
 	{
-#if PG_VERSION_NUM < 90600
-		Oid			umoid;
-#endif
-
 		entry->conn = mysql_connect(opt);
 		elog(DEBUG3, "new mysql_fdw connection %p for server \"%s\"",
 			 entry->conn, server->servername);
@@ -137,25 +132,10 @@ mysql_get_connection(ForeignServer *server, UserMapping *user, mysql_opt *opt)
 		entry->server_hashvalue =
 			GetSysCacheHashValue1(FOREIGNSERVEROID,
 								  ObjectIdGetDatum(server->serverid));
-#if PG_VERSION_NUM >= 90600
+
 		entry->mapping_hashvalue =
 			GetSysCacheHashValue1(USERMAPPINGOID,
 								  ObjectIdGetDatum(user->umid));
-#else
-		/* Pre-9.6, UserMapping doesn't store its OID, so look it up again */
-		umoid = GetSysCacheOid2(USERMAPPINGUSERSERVER,
-								ObjectIdGetDatum(user->userid),
-								ObjectIdGetDatum(user->serverid));
-		if (!OidIsValid(umoid))
-		{
-			/* Not found for the specific user -- try PUBLIC */
-			umoid = GetSysCacheOid2(USERMAPPINGUSERSERVER,
-									ObjectIdGetDatum(InvalidOid),
-									ObjectIdGetDatum(user->serverid));
-		}
-		entry->mapping_hashvalue =
-			GetSysCacheHashValue1(USERMAPPINGOID, ObjectIdGetDatum(umoid));
-#endif
 	}
 	return entry->conn;
 }
@@ -191,7 +171,7 @@ mysql_cleanup_connection(void)
 void
 mysql_release_connection(MYSQL *conn)
 {
-	HASH_SEQ_STATUS	scan;
+	HASH_SEQ_STATUS scan;
 	ConnCacheEntry *entry;
 
 	if (ConnectionHash == NULL)
@@ -233,7 +213,7 @@ mysql_connect(mysql_opt *opt)
 				(errcode(ERRCODE_FDW_OUT_OF_MEMORY),
 				 errmsg("failed to initialise the MySQL connection object")));
 
-	mysql_options(conn, MYSQL_SET_CHARSET_NAME, GetDatabaseEncodingName());
+	mysql_options(conn, MYSQL_SET_CHARSET_NAME, opt->character_set);
 #if MYSQL_VERSION_ID < 80000
 	mysql_options(conn, MYSQL_SECURE_AUTH, &secure_auth);
 #endif
@@ -245,17 +225,17 @@ mysql_connect(mysql_opt *opt)
 		mysql_options(conn, MYSQL_INIT_COMMAND, svr_init_command);
 
 	/*
-	 * Enable or disable automatic reconnection to the MySQL server if
-	 * the existing connection is found to have been lost.
+	 * Enable or disable automatic reconnection to the MySQL server if the
+	 * existing connection is found to have been lost.
 	 */
 	mysql_options(conn, MYSQL_OPT_RECONNECT, &opt->reconnect);
 
 	mysql_ssl_set(conn, opt->ssl_key, opt->ssl_cert, opt->ssl_ca,
-				   opt->ssl_capath, ssl_cipher);
+				  opt->ssl_capath, ssl_cipher);
 
 	if (!mysql_real_connect(conn, opt->svr_address, opt->svr_username,
-							 opt->svr_password, svr_database, opt->svr_port,
-							 NULL, 0))
+							opt->svr_password, svr_database, opt->svr_port,
+							NULL, 0))
 		ereport(ERROR,
 				(errcode(ERRCODE_FDW_UNABLE_TO_ESTABLISH_CONNECTION),
 				 errmsg("failed to connect to MySQL: %s", mysql_error(conn))));
