@@ -4,7 +4,7 @@
  * 		Query deparser for mysql_fdw
  *
  * Portions Copyright (c) 2012-2014, PostgreSQL Global Development Group
- * Portions Copyright (c) 2004-2021, EnterpriseDB Corporation.
+ * Portions Copyright (c) 2004-2022, EnterpriseDB Corporation.
  *
  * IDENTIFICATION
  * 		deparse.c
@@ -61,7 +61,7 @@ typedef struct foreign_glob_cxt
 	 * of join conditions. Also true for aggregate relations to restrict
 	 * aggregates for specified list.
 	 */
-	bool		is_remote_cond;	/* true for join or aggregate relations */
+	bool		is_remote_cond; /* true for join or aggregate relations */
 	Relids		relids;			/* relids of base relations in the underlying
 								 * scan */
 } foreign_glob_cxt;
@@ -153,12 +153,13 @@ static void mysql_deparse_from_expr_for_rel(StringInfo buf, PlannerInfo *root,
 static void mysql_deparse_from_expr(List *quals, deparse_expr_cxt *context);
 static void mysql_append_function_name(Oid funcid, deparse_expr_cxt *context);
 static void mysql_deparse_aggref(Aggref *node, deparse_expr_cxt *context);
-#if PG_VERSION_NUM >= 100000
 static void mysql_append_groupby_clause(List *tlist, deparse_expr_cxt *context);
 static Node *mysql_deparse_sort_group_clause(Index ref, List *tlist,
 											 bool force_colno,
 											 deparse_expr_cxt *context);
-#endif
+static void mysql_append_orderby_clause(List *pathkeys, bool has_final_sort,
+										deparse_expr_cxt *context);
+static void mysql_append_limit_clause(deparse_expr_cxt *context);
 
 /*
  * Functions to construct string representation of a specific types.
@@ -242,6 +243,8 @@ mysql_quote_identifier(const char *str, char quotechar)
  *
  * remote_conds is the list of conditions to be deparsed into the WHERE clause.
  *
+ * pathkeys is the list of pathkeys to order the result by.
+ *
  * If params_list is not NULL, it receives a list of Params and other-relation
  * Vars used in the clauses; these values must be transmitted to the remote
  * server as parameter values.
@@ -254,38 +257,26 @@ mysql_quote_identifier(const char *str, char quotechar)
 extern void
 mysql_deparse_select_stmt_for_rel(StringInfo buf, PlannerInfo *root,
 								  RelOptInfo *rel, List *tlist,
-								  List *remote_conds, List **retrieved_attrs,
-								  List **params_list)
+								  List *remote_conds, List *pathkeys,
+								  bool has_final_sort, bool has_limit,
+								  List **retrieved_attrs, List **params_list)
 {
 	deparse_expr_cxt context;
 	List	   *quals;
-#if PG_VERSION_NUM >= 100000
 	MySQLFdwRelationInfo *fpinfo = (MySQLFdwRelationInfo *) rel->fdw_private;
-#endif
 
 	/*
-	 * We handle relations for foreign tables and joins between those and upper
-	 * relations.
+	 * We handle relations for foreign tables and joins between those and
+	 * upper relations.
 	 */
-#if PG_VERSION_NUM >= 100000
 	Assert(IS_JOIN_REL(rel) || IS_SIMPLE_REL(rel) || IS_UPPER_REL(rel));
-#else
-	Assert(rel->reloptkind == RELOPT_JOINREL ||
-		   rel->reloptkind == RELOPT_BASEREL ||
-		   rel->reloptkind == RELOPT_UPPER_REL ||
-		   rel->reloptkind == RELOPT_OTHER_MEMBER_REL);
-#endif
 
 	/* Fill portions of context common to base relation */
 	context.buf = buf;
 	context.root = root;
 	context.foreignrel = rel;
 	context.params_list = params_list;
-#if PG_VERSION_NUM >= 100000
 	context.scanrel = IS_UPPER_REL(rel) ? fpinfo->outerrel : rel;
-#else
-	context.scanrel = rel;
-#endif
 
 	/* Construct SELECT clause */
 	mysql_deparse_select_sql(tlist, retrieved_attrs, &context);
@@ -295,7 +286,6 @@ mysql_deparse_select_stmt_for_rel(StringInfo buf, PlannerInfo *root,
 	 * conditions of the underlying scan relation; otherwise, we can use the
 	 * supplied list of remote conditions directly.
 	 */
-#if PG_VERSION_NUM >= 100000
 	if (IS_UPPER_REL(rel))
 	{
 		MySQLFdwRelationInfo *ofpinfo;
@@ -304,13 +294,11 @@ mysql_deparse_select_stmt_for_rel(StringInfo buf, PlannerInfo *root,
 		quals = ofpinfo->remote_conds;
 	}
 	else
-#endif
 		quals = remote_conds;
 
 	/* Construct FROM and WHERE clauses */
 	mysql_deparse_from_expr(quals, &context);
 
-#if PG_VERSION_NUM >= 100000
 	if (IS_UPPER_REL(rel))
 	{
 		/* Append GROUP BY clause */
@@ -323,7 +311,14 @@ mysql_deparse_select_stmt_for_rel(StringInfo buf, PlannerInfo *root,
 			mysql_append_conditions(remote_conds, &context);
 		}
 	}
-#endif
+
+	/* Add ORDER BY clause if we found any useful pathkeys */
+	if (pathkeys)
+		mysql_append_orderby_clause(pathkeys, has_final_sort, &context);
+
+	/* Add LIMIT clause if necessary */
+	if (has_limit)
+		mysql_append_limit_clause(&context);
 }
 
 /*
@@ -351,12 +346,7 @@ mysql_deparse_select_sql(List *tlist, List **retrieved_attrs,
 	 */
 	appendStringInfoString(buf, "SELECT ");
 
-#if PG_VERSION_NUM >= 100000
 	if (IS_JOIN_REL(foreignrel) || IS_UPPER_REL(foreignrel))
-#else
-	if (foreignrel->reloptkind == RELOPT_JOINREL ||
-		foreignrel->reloptkind == RELOPT_UPPER_REL)
-#endif
 	{
 		/*
 		 * For a join or upper relation the input tlist gives the list of
@@ -371,8 +361,8 @@ mysql_deparse_select_sql(List *tlist, List **retrieved_attrs,
 		MySQLFdwRelationInfo *fpinfo = (MySQLFdwRelationInfo *) foreignrel->fdw_private;
 
 		/*
-		 * Core code already has some lock on each rel being planned, so we can
-		 * use NoLock here.
+		 * Core code already has some lock on each rel being planned, so we
+		 * can use NoLock here.
 		 */
 #if PG_VERSION_NUM < 130000
 		rel = heap_open(rte->relid, NoLock);
@@ -436,10 +426,8 @@ mysql_deparse_from_expr(List *quals, deparse_expr_cxt *context)
 	RelOptInfo *scanrel = context->scanrel;
 
 	/* For upper relations, scanrel must be either a joinrel or a baserel */
-#if PG_VERSION_NUM >= 100000
 	Assert(!IS_UPPER_REL(context->foreignrel) ||
 		   IS_JOIN_REL(scanrel) || IS_SIMPLE_REL(scanrel));
-#endif
 
 	/* Construct FROM clause */
 	appendStringInfoString(buf, " FROM ");
@@ -602,8 +590,8 @@ mysql_deparse_column_ref(StringInfo buf, int varno, int varattno,
 	}
 
 	/*
-	 * If it's a column of a regular table or it doesn't have column_name
-	 * FDW option, use attribute name.
+	 * If it's a column of a regular table or it doesn't have column_name FDW
+	 * option, use attribute name.
 	 */
 	if (colname == NULL)
 #if PG_VERSION_NUM >= 110000
@@ -627,7 +615,7 @@ mysql_deparse_string(StringInfo buf, const char *val, bool isstr)
 	if (isstr)
 		appendStringInfoChar(buf, '\'');
 
-	for (valptr = val; *valptr; valptr++,i++)
+	for (valptr = val; *valptr; valptr++, i++)
 	{
 		char		ch = *valptr;
 
@@ -752,6 +740,10 @@ deparse_interval(StringInfo buf, Datum datum)
 	struct pg_tm tm;
 	fsec_t		fsec;
 	bool		is_first = true;
+#if PG_VERSION_NUM >= 150000
+	struct pg_itm tt,
+			   *itm = &tt;
+#endif
 
 #define append_interval(expr, unit) \
 do { \
@@ -764,8 +756,19 @@ do { \
 	/* Check saved opname. It could be only "+" and "-" */
 	Assert(cur_opname);
 
+#if PG_VERSION_NUM >= 150000
+	interval2itm(*DatumGetIntervalP(datum), itm);
+	tm.tm_sec = itm->tm_sec;
+	tm.tm_min = itm->tm_min;
+	tm.tm_hour = itm->tm_hour;
+	tm.tm_mday = itm->tm_mday;
+	tm.tm_mon = itm->tm_mon;
+	tm.tm_year = itm->tm_year;
+	fsec = itm->tm_usec;
+#else
 	if (interval2tm(*DatumGetIntervalP(datum), &tm, &fsec) != 0)
 		elog(ERROR, "could not convert interval to tm");
+#endif
 
 	if (tm.tm_year > 0)
 		append_interval(tm.tm_year, "YEAR");
@@ -969,6 +972,7 @@ mysql_deparse_const(Const *node, deparse_expr_cxt *context)
 			deparse_interval(buf, node->constvalue);
 			break;
 		case BYTEAOID:
+
 			/*
 			 * The string for BYTEA always seems to be in the format "\\x##"
 			 * where # is a hex digit, Even if the value passed in is
@@ -1468,7 +1472,11 @@ mysql_print_remote_placeholder(Oid paramtype, int32 paramtypmod,
 static bool
 is_builtin(Oid oid)
 {
+#if PG_VERSION_NUM >= 150000
+	return (oid < FirstGenbkiObjectId);
+#else
 	return (oid < FirstBootstrapObjectId);
+#endif
 }
 
 /*
@@ -1669,7 +1677,8 @@ foreign_expr_walker(Node *node, foreign_glob_cxt *glob_cxt,
 				const char *operatorName = get_opname(oe->opno);
 
 				/*
-				 * Join-pushdown allows only a few operators to be pushed down.
+				 * Join-pushdown allows only a few operators to be pushed
+				 * down.
 				 */
 				if (glob_cxt->is_remote_cond &&
 					(!(strcmp(operatorName, "<") == 0 ||
@@ -1875,7 +1884,6 @@ foreign_expr_walker(Node *node, foreign_glob_cxt *glob_cxt,
 				ListCell   *lc;
 				const char *func_name = get_func_name(agg->aggfnoid);
 
-#if PG_VERSION_NUM >= 100000
 				/* Not safe to pushdown when not in grouping context */
 				if (!IS_UPPER_REL(glob_cxt->foreignrel))
 					return false;
@@ -1883,20 +1891,18 @@ foreign_expr_walker(Node *node, foreign_glob_cxt *glob_cxt,
 				/* Only non-split aggregates are pushable. */
 				if (agg->aggsplit != AGGSPLIT_SIMPLE)
 					return false;
-#endif
 
 				/* As usual, it must be shippable. */
 				if (!is_builtin(agg->aggfnoid))
 					return false;
 
 				if (!(strcmp(func_name, "min") == 0 ||
-					strcmp(func_name, "max") == 0 ||
-					strcmp(func_name, "sum") == 0 ||
-					strcmp(func_name, "avg") == 0 ||
-					strcmp(func_name, "count") == 0))
+					  strcmp(func_name, "max") == 0 ||
+					  strcmp(func_name, "sum") == 0 ||
+					  strcmp(func_name, "avg") == 0 ||
+					  strcmp(func_name, "count") == 0))
 					return false;
 
-
 				/*
 				 * Recurse to input args. aggdirectargs, aggorder and
 				 * aggdistinct are all present in args, so no need to check
@@ -1926,6 +1932,10 @@ foreign_expr_walker(Node *node, foreign_glob_cxt *glob_cxt,
 				if (agg->aggfilter)
 					return false;
 
+				/* VARIADIC not supported on MySQL. */
+				if (agg->aggvariadic)
+					return false;
+
 				/*
 				 * If aggregate's input collation is not derived from a
 				 * foreign Var, it can't be sent to remote.
@@ -2028,9 +2038,7 @@ mysql_is_foreign_expr(PlannerInfo *root, RelOptInfo *baserel, Expr *expr,
 {
 	foreign_glob_cxt glob_cxt;
 	foreign_loc_cxt loc_cxt;
-#if PG_VERSION_NUM >= 100000
 	MySQLFdwRelationInfo *fpinfo = (MySQLFdwRelationInfo *) (baserel->fdw_private);
-#endif
 
 	/*
 	 * Check that the expression consists of nodes that are safe to execute
@@ -2045,11 +2053,9 @@ mysql_is_foreign_expr(PlannerInfo *root, RelOptInfo *baserel, Expr *expr,
 	 * because the upperrel's own relids currently aren't set to anything
 	 * meaningful by the core code.  For other relation, use their own relids.
 	 */
-#if PG_VERSION_NUM >= 100000
 	if (IS_UPPER_REL(baserel))
 		glob_cxt.relids = fpinfo->outerrel->relids;
 	else
-#endif
 		glob_cxt.relids = baserel->relids;
 
 	loc_cxt.collation = InvalidOid;
@@ -2133,11 +2139,7 @@ mysql_deparse_from_expr_for_rel(StringInfo buf, PlannerInfo *root,
 {
 	MySQLFdwRelationInfo *fpinfo = (MySQLFdwRelationInfo *) foreignrel->fdw_private;
 
-#if PG_VERSION_NUM >= 100000
 	if (IS_JOIN_REL(foreignrel))
-#else
-	if (foreignrel->reloptkind == RELOPT_JOINREL)
-#endif
 	{
 		RelOptInfo *rel_o = fpinfo->outerrel;
 		RelOptInfo *rel_i = fpinfo->innerrel;
@@ -2190,8 +2192,8 @@ mysql_deparse_from_expr_for_rel(StringInfo buf, PlannerInfo *root,
 		Relation	rel;
 
 		/*
-		 * Core code already has some lock on each rel being planned, so we can
-		 * use NoLock here.
+		 * Core code already has some lock on each rel being planned, so we
+		 * can use NoLock here.
 		 */
 #if PG_VERSION_NUM < 130000
 		rel = heap_open(rte->relid, NoLock);
@@ -2282,14 +2284,10 @@ static void
 mysql_deparse_aggref(Aggref *node, deparse_expr_cxt *context)
 {
 	StringInfo	buf = context->buf;
-	bool		use_variadic;
 
 	/* Only basic, non-split aggregation accepted. */
 	Assert(node->aggsplit == AGGSPLIT_SIMPLE);
 
-	/* Check if need to print VARIADIC (cf. ruleutils.c) */
-	use_variadic = node->aggvariadic;
-
 	/* Find aggregate name from aggfnoid which is a pg_proc entry. */
 	mysql_append_function_name(node->aggfnoid, context);
 	appendStringInfoChar(buf, '(');
@@ -2318,14 +2316,6 @@ mysql_deparse_aggref(Aggref *node, deparse_expr_cxt *context)
 				appendStringInfoString(buf, ", ");
 			first = false;
 
-			/* Add VARIADIC */
-#if PG_VERSION_NUM >= 130000
-			if (use_variadic && lnext(node->args, arg) == NULL)
-#else
-			if (use_variadic && lnext(arg) == NULL)
-#endif
-				appendStringInfoString(buf, "VARIADIC ");
-
 			deparseExpr((Expr *) n, context);
 		}
 	}
@@ -2337,7 +2327,6 @@ mysql_deparse_aggref(Aggref *node, deparse_expr_cxt *context)
  * mysql_append_groupby_clause
  * 		Deparse GROUP BY clause.
  */
-#if PG_VERSION_NUM >= 100000
 static void
 mysql_append_groupby_clause(List *tlist, deparse_expr_cxt *context)
 {
@@ -2413,7 +2402,6 @@ mysql_deparse_sort_group_clause(Index ref, List *tlist, bool force_colno,
 
 	return (Node *) expr;
 }
-#endif
 
 /*
  * mysql_is_foreign_param
@@ -2443,13 +2431,11 @@ mysql_is_foreign_param(PlannerInfo *root,
 				/* It would have to be sent unless it's a foreign Var. */
 				Var		   *var = (Var *) expr;
 				Relids		relids;
-#if PG_VERSION_NUM >= 100000
 				MySQLFdwRelationInfo *fpinfo = (MySQLFdwRelationInfo *) (baserel->fdw_private);
 
 				if (IS_UPPER_REL(baserel))
 					relids = fpinfo->outerrel->relids;
 				else
-#endif
 					relids = baserel->relids;
 
 				if (bms_is_member(var->varno, relids) && var->varlevelsup == 0)
@@ -2466,3 +2452,84 @@ mysql_is_foreign_param(PlannerInfo *root,
 	}
 	return false;
 }
+
+/*
+ * mysql_append_orderby_clause
+ * 		Deparse ORDER BY clause according to the given pathkeys for given
+ * 		base relation. From given pathkeys expressions belonging entirely
+ * 		to the given base relation are obtained and deparsed.
+ */
+void
+mysql_append_orderby_clause(List *pathkeys, bool has_final_sort,
+							deparse_expr_cxt *context)
+{
+	ListCell   *lcell;
+	char	   *delim = " ";
+	RelOptInfo *baserel = context->scanrel;
+	StringInfo	buf = context->buf;
+
+	appendStringInfo(buf, " ORDER BY");
+	foreach(lcell, pathkeys)
+	{
+		PathKey    *pathkey = lfirst(lcell);
+		Expr	   *em_expr;
+
+		if (has_final_sort)
+		{
+			/*
+			 * By construction, context->foreignrel is the input relation to
+			 * the final sort.
+			 */
+			em_expr = mysql_find_em_expr_for_input_target(context->root,
+														  pathkey->pk_eclass,
+														  context->foreignrel->reltarget);
+		}
+		else
+			em_expr = mysql_find_em_expr_for_rel(pathkey->pk_eclass, baserel);
+
+		Assert(em_expr != NULL);
+
+		appendStringInfoString(buf, delim);
+		deparseExpr(em_expr, context);
+
+		if (pathkey->pk_nulls_first)\
+			appendStringInfoString(buf, " IS NOT NULL");
+		else
+			appendStringInfoString(buf, " IS NULL");
+
+		/* Add delimiter */
+		appendStringInfoString(buf, ", ");
+
+		deparseExpr(em_expr, context);
+
+		if (pathkey->pk_strategy == BTLessStrategyNumber)
+			appendStringInfoString(buf, " ASC");
+		else
+			appendStringInfoString(buf, " DESC");
+
+		delim = ", ";
+	}
+}
+
+/*
+ * mysql_append_limit_clause
+ * 		Deparse LIMIT OFFSET clause.
+ */
+static void
+mysql_append_limit_clause(deparse_expr_cxt *context)
+{
+	PlannerInfo *root = context->root;
+	StringInfo	buf = context->buf;
+
+	if (root->parse->limitCount)
+	{
+		appendStringInfoString(buf, " LIMIT ");
+		deparseExpr((Expr *) root->parse->limitCount, context);
+
+		if (root->parse->limitOffset)
+		{
+			appendStringInfoString(buf, " OFFSET ");
+			deparseExpr((Expr *) root->parse->limitOffset, context);
+		}
+	}
+}
