@@ -4,7 +4,7 @@
  * 		Foreign-data wrapper for remote MySQL servers
  *
  * Portions Copyright (c) 2012-2014, PostgreSQL Global Development Group
- * Portions Copyright (c) 2004-2021, EnterpriseDB Corporation.
+ * Portions Copyright (c) 2004-2022, EnterpriseDB Corporation.
  *
  * IDENTIFICATION
  * 		mysql_fdw.h
@@ -78,9 +78,9 @@
 
 /* Macro for list API backporting. */
 #if PG_VERSION_NUM < 130000
-	#define mysql_list_concat(l1, l2) list_concat(l1, list_copy(l2))
+#define mysql_list_concat(l1, l2) list_concat(l1, list_copy(l2))
 #else
-	#define mysql_list_concat(l1, l2) list_concat((l1), (l2))
+#define mysql_list_concat(l1, l2) list_concat((l1), (l2))
 #endif
 
 /*
@@ -103,6 +103,8 @@ typedef struct mysql_opt
 	bool		use_remote_estimate;	/* use remote estimate for rows */
 	unsigned long fetch_size;	/* Number of rows to fetch from remote server */
 	bool		reconnect;		/* set to true for automatic reconnection */
+	char	   *character_set;	/* Character set used for remote connection */
+	char	   *sql_mode;		/* MySQL sql_mode variable for connection */
 
 	/* SSL parameters; unused options may be given as NULL */
 	char	   *ssl_key;		/* MySQL SSL: path to the key file */
@@ -171,7 +173,7 @@ typedef struct MySQLFdwExecState
 	mysql_table *table;
 	char	   *query;			/* Query string */
 	List	   *retrieved_attrs;	/* list of target attribute numbers */
-	bool		query_executed;	/* have we executed the query? */
+	bool		query_executed; /* have we executed the query? */
 	int			numParams;		/* number of parameters passed to query */
 	FmgrInfo   *param_flinfo;	/* output conversion functions for them */
 	List	   *param_exprs;	/* executable expressions for param values */
@@ -183,22 +185,23 @@ typedef struct MySQLFdwExecState
 	MemoryContext temp_cxt;		/* context for per-tuple temporary data */
 	AttInMetadata *attinmeta;
 	AttrNumber	rowidAttno;		/* attnum of resjunk rowid column */
+	bool		has_var_size_col;	/* true if fetching var size columns */
 
 	/*
 	 * Members used for constructing the ForeignScan result row when whole-row
 	 * references are involved in a pushed down join.
 	 */
-	MySQLWRState **mysqlwrstates; /* whole-row construction information for
-								   * each base relation involved in the pushed
-								   * down join. */
+	MySQLWRState **mysqlwrstates;	/* whole-row construction information for
+									 * each base relation involved in the
+									 * pushed down join. */
 	int		   *wr_attrs_pos;	/* Array mapping the attributes in the
 								 * ForeignScan result to those in the rows
 								 * fetched from the foreign server.  The array
 								 * is indexed by the attribute numbers in the
 								 * ForeignScan. */
 	TupleDesc	wr_tupdesc;		/* Tuple descriptor describing the result of
-								 * ForeignScan node.  Should be same as that in
-								 * ForeignScanState::ss::ss_ScanTupleSlot */
+								 * ForeignScan node.  Should be same as that
+								 * in ForeignScanState::ss::ss_ScanTupleSlot */
 	/* Array for holding column values. */
 	Datum	   *wr_values;
 	bool	   *wr_nulls;
@@ -219,6 +222,9 @@ typedef struct MySQLFdwRelationInfo
 	/* Bitmap of attr numbers we need to fetch from the remote server. */
 	Bitmapset  *attrs_used;
 
+	/* True means that the query_pathkeys is safe to push down */
+	bool		qp_is_pushdown_safe;
+
 	/*
 	 * Name of the relation while EXPLAINing ForeignScan.  It is used for join
 	 * relations but is set for all relations.  For join relation, the name
@@ -233,6 +239,9 @@ typedef struct MySQLFdwRelationInfo
 	List	   *joinclauses;
 	/* Grouping information */
 	List	   *grouped_tlist;
+
+	/* Upper relation information */
+	UpperRelationKind stage;
 } MySQLFdwRelationInfo;
 
 
@@ -263,7 +272,7 @@ extern bool ((mysql_stmt_bind_result) (MYSQL_STMT *stmt, MYSQL_BIND *bnd));
 
 extern MYSQL_STMT *((mysql_stmt_init) (MYSQL *mysql));
 extern MYSQL_RES *((mysql_stmt_result_metadata) (MYSQL_STMT *stmt));
-extern int ((mysql_stmt_store_result) (MYSQL *mysql));
+extern int ((mysql_stmt_store_result) (MYSQL_STMT *stmt));
 extern MYSQL_ROW((mysql_fetch_row) (MYSQL_RES *result));
 extern MYSQL_FIELD *((mysql_fetch_field) (MYSQL_RES *result));
 extern MYSQL_FIELD *((mysql_fetch_fields) (MYSQL_RES *result));
@@ -310,11 +319,18 @@ extern void mysql_deparse_select_stmt_for_rel(StringInfo buf,
 											  PlannerInfo *root,
 											  RelOptInfo *rel, List *tlist,
 											  List *remote_conds,
+											  List *pathkeys,
+											  bool has_final_sort,
+											  bool has_limit,
 											  List **retrieved_attrs,
 											  List **params_list);
 extern const char *mysql_get_jointype_name(JoinType jointype);
 extern bool mysql_is_foreign_param(PlannerInfo *root, RelOptInfo *baserel,
 								   Expr *expr);
+extern Expr *mysql_find_em_expr_for_rel(EquivalenceClass *ec, RelOptInfo *rel);
+extern Expr *mysql_find_em_expr_for_input_target(PlannerInfo *root,
+												 EquivalenceClass *ec,
+												 PathTarget *target);
 
 /* connection.c headers */
 MYSQL *mysql_get_connection(ForeignServer *server, UserMapping *user,
