@@ -4,7 +4,7 @@
  * 		Type handling for remote MySQL servers
  *
  * Portions Copyright (c) 2012-2014, PostgreSQL Global Development Group
- * Portions Copyright (c) 2004-2021, EnterpriseDB Corporation.
+ * Portions Copyright (c) 2004-2022, EnterpriseDB Corporation.
  *
  * IDENTIFICATION
  * 		mysql_query.c
@@ -64,9 +64,9 @@ mysql_convert_to_pg(Oid pgtyp, int pgtypmod, mysql_column *column)
 	Datum		valueDatum;
 	regproc		typeinput;
 	HeapTuple	tuple;
-	int			typemod;
 	char		str[MAXDATELEN];
 	bytea	   *result;
+	char	   *text_result = NULL;
 
 	/* get the type's output function */
 	tuple = SearchSysCache1(TYPEOID, ObjectIdGetDatum(pgtyp));
@@ -74,7 +74,6 @@ mysql_convert_to_pg(Oid pgtyp, int pgtypmod, mysql_column *column)
 		elog(ERROR, "cache lookup failed for type%u", pgtyp);
 
 	typeinput = ((Form_pg_type) GETSTRUCT(tuple))->typinput;
-	typemod = ((Form_pg_type) GETSTRUCT(tuple))->typtypmod;
 	ReleaseSysCache(tuple);
 
 	switch (pgtyp)
@@ -109,13 +108,24 @@ mysql_convert_to_pg(Oid pgtyp, int pgtypmod, mysql_column *column)
 			sprintf(str, "%d", dec_bin(*((int *) column->value)));
 			valueDatum = CStringGetDatum((char *) str);
 			break;
+
+		case TEXTOID:
+			text_result = (char *) palloc(column->length + 1);
+			memcpy(text_result, (char *) column->value, column->length);
+			text_result[column->length] = '\0';
+			valueDatum = CStringGetDatum((char *) text_result);
+			break;
+
 		default:
 			valueDatum = CStringGetDatum((char *) column->value);
 	}
 
 	value_datum = OidFunctionCall3(typeinput, valueDatum,
 								   ObjectIdGetDatum(pgtyp),
-								   Int32GetDatum(typemod));
+								   Int32GetDatum(pgtypmod));
+
+	if (text_result)
+		pfree(text_result);
 
 	return value_datum;
 }
@@ -408,6 +418,8 @@ mysql_bind_result(Oid pgtyp, int pgtypmod, MYSQL_FIELD *field,
 {
 	MYSQL_BIND *mbind = column->mysql_bind;
 
+	memset(mbind, 0, sizeof(MYSQL_BIND));
+
 #if MYSQL_VERSION_ID < 80000 || MARIADB_VERSION_ID >= 100000
 	mbind->is_null = (my_bool *) &column->is_null;
 	mbind->error = (my_bool *) &column->error;
@@ -426,6 +438,20 @@ mysql_bind_result(Oid pgtyp, int pgtypmod, MYSQL_FIELD *field,
 			mbind->buffer = VARDATA(column->value);
 			mbind->buffer_length = MAX_BLOB_WIDTH;
 			break;
+		case TEXTOID:
+			mbind->buffer_type = MYSQL_TYPE_VAR_STRING;
+			if (field->max_length == 0)
+			{
+				column->value = (Datum) palloc0(MAXDATALEN);
+				mbind->buffer_length = MAXDATALEN;
+			}
+			else
+			{
+				column->value = (Datum) palloc0(field->max_length);
+				mbind->buffer_length = field->max_length;
+			}
+			mbind->buffer = (char *) column->value;
+			break;
 		default:
 			mbind->buffer_type = MYSQL_TYPE_VAR_STRING;
 			column->value = (Datum) palloc0(MAXDATALEN);
