@@ -4,7 +4,7 @@
  * 		FDW option handling for mysql_fdw
  *
  * Portions Copyright (c) 2012-2014, PostgreSQL Global Development Group
- * Portions Copyright (c) 2004-2021, EnterpriseDB Corporation.
+ * Portions Copyright (c) 2004-2022, EnterpriseDB Corporation.
  *
  * IDENTIFICATION
  * 		option.c
@@ -19,6 +19,7 @@
 #include "catalog/pg_user_mapping.h"
 #include "catalog/pg_type.h"
 #include "commands/defrem.h"
+#include "mb/pg_wchar.h"
 #include "miscadmin.h"
 #include "mysql_fdw.h"
 #include "utils/lsyscache.h"
@@ -52,6 +53,8 @@ static struct MySQLFdwOption valid_options[] =
 	{"fetch_size", ForeignServerRelationId},
 	{"fetch_size", ForeignTableRelationId},
 	{"reconnect", ForeignServerRelationId},
+	{"character_set", ForeignServerRelationId},
+	{"sql_mode", ForeignServerRelationId},
 	{"ssl_key", ForeignServerRelationId},
 	{"ssl_cert", ForeignServerRelationId},
 	{"ssl_ca", ForeignServerRelationId},
@@ -114,11 +117,11 @@ mysql_fdw_validator(PG_FUNCTION_ARGS)
 		/* Validate fetch_size option value */
 		if (strcmp(def->defname, "fetch_size") == 0)
 		{
-			unsigned long	fetch_size;
+			unsigned long fetch_size;
 			char	   *endptr;
 			char	   *inputVal = defGetString(def);
 
-			while(inputVal && isspace((unsigned char) *inputVal))
+			while (inputVal && isspace((unsigned char) *inputVal))
 				inputVal++;
 
 			if (inputVal && *inputVal == '-')
@@ -252,6 +255,12 @@ mysql_get_options(Oid foreignoid, bool is_foreigntable)
 		if (strcmp(def->defname, "reconnect") == 0)
 			opt->reconnect = defGetBoolean(def);
 
+		if (strcmp(def->defname, "character_set") == 0)
+			opt->character_set = defGetString(def);
+
+		if (strcmp(def->defname, "sql_mode") == 0)
+			opt->sql_mode = defGetString(def);
+
 		if (strcmp(def->defname, "ssl_key") == 0)
 			opt->ssl_key = defGetString(def);
 
@@ -277,8 +286,8 @@ mysql_get_options(Oid foreignoid, bool is_foreigntable)
 
 	/*
 	 * When we don't have a table name or database name provided in the
-	 * FOREIGN TABLE options, then use a foreign table name as the target table
-	 * name and the namespace of the foreign table as a database name.
+	 * FOREIGN TABLE options, then use a foreign table name as the target
+	 * table name and the namespace of the foreign table as a database name.
 	 */
 	if (f_table)
 	{
@@ -293,5 +302,16 @@ mysql_get_options(Oid foreignoid, bool is_foreigntable)
 	if (!opt->fetch_size)
 		opt->fetch_size = MYSQL_PREFETCH_ROWS;
 
+	/* Default value for character_set */
+	if (!opt->character_set)
+		opt->character_set = MYSQL_AUTODETECT_CHARSET_NAME;
+	/* Special value provided for existing behavior */
+	else if (strcmp(opt->character_set, "PGDatabaseEncoding") == 0)
+		opt->character_set = (char *) GetDatabaseEncodingName();
+
+	/* Default value for sql_mode */
+	if (!opt->sql_mode)
+		opt->sql_mode = "ANSI_QUOTES";
+
 	return opt;
 }
