@@ -234,6 +234,26 @@ EXPLAIN (VERBOSE, COSTS OFF)
 SELECT c2, least_agg(c1) FROM fdw132_t1 GROUP BY c2 ORDER BY c2;
 SELECT c2, least_agg(c1) FROM fdw132_t1 GROUP BY c2 ORDER BY c2;
 
+-- FDW-129: Limit and offset pushdown with Aggregate pushdown.
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT min(c1), c1 FROM fdw132_t1 GROUP BY c1 ORDER BY c1 LIMIT 1 OFFSET 1;
+SELECT min(c1), c1 FROM fdw132_t1 GROUP BY c1 ORDER BY c1 LIMIT 1 OFFSET 1;
+
+-- Limit 0, Offset 0 with aggregates.
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT min(c1), c1 FROM fdw132_t1 GROUP BY c1 ORDER BY c1 LIMIT 0 OFFSET 0;
+SELECT min(c1), c1 FROM fdw132_t1 GROUP BY c1 ORDER BY c1 LIMIT 0 OFFSET 0;
+
+-- Limit NULL
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT min(c1), c1 FROM fdw132_t1 GROUP BY c1 ORDER BY c1 LIMIT NULL OFFSET 2;
+SELECT min(c1), c1 FROM fdw132_t1 GROUP BY c1 ORDER BY c1 LIMIT NULL OFFSET 2;
+
+-- Limit ALL
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT min(c1), c1 FROM fdw132_t1 GROUP BY c1 ORDER BY c1 LIMIT ALL OFFSET 2;
+SELECT min(c1), c1 FROM fdw132_t1 GROUP BY c1 ORDER BY c1 LIMIT ALL OFFSET 2;
+
 -- Delete existing data and load new data for partition-wise aggregate test
 -- cases.
 DELETE FROM fdw132_t1;
@@ -246,19 +266,8 @@ INSERT INTO fdw132_t2 values(4, 4, 'AAA12', 'foo');
 -- Test partition-wise aggregates
 SET enable_partitionwise_aggregate TO on;
 
--- Create the partition table in plpgsql block as those are failing with
--- different error messages on back-branches.
--- All test cases related to partition-wise join gives an error on v96 as
--- partition syntax is not supported there.
-DO
-$$
-BEGIN
-  EXECUTE 'CREATE TABLE fprt1 (c1 int, c2 int, c3 varchar, c4 varchar) PARTITION BY RANGE(c1)';
-EXCEPTION WHEN others THEN
-  RAISE NOTICE 'syntax error';
-END;
-$$
-LANGUAGE plpgsql;
+-- Create the partition table.
+CREATE TABLE fprt1 (c1 int, c2 int, c3 varchar, c4 varchar) PARTITION BY RANGE(c1);
 CREATE FOREIGN TABLE ftprt1_p1 PARTITION OF fprt1 FOR VALUES FROM (1) TO (2)
   SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'test1');
 CREATE FOREIGN TABLE ftprt1_p2 PARTITION OF fprt1 FOR VALUES FROM (3) TO (4)
