@@ -1,11 +1,11 @@
-\set MYSQL_HOST			'\'localhost\''
-\set MYSQL_PORT			'\'3306\''
-\set MYSQL_USER_NAME	'\'edb\''
-\set MYSQL_PASS			'\'edb\''
+\set MYSQL_HOST			`echo \'"$MYSQL_HOST"\'`
+\set MYSQL_PORT			`echo \'"$MYSQL_PORT"\'`
+\set MYSQL_USER_NAME	`echo \'"$MYSQL_USER_NAME"\'`
+\set MYSQL_PASS			`echo \'"$MYSQL_PWD"\'`
 
 -- Before running this file User must create database mysql_fdw_regress on
--- MySQL with all permission for 'edb' user with 'edb' password and ran
--- mysql_init.sh file to create tables.
+-- MySQL with all permission for MYSQL_USER_NAME user with MYSQL_PWD password
+-- and ran mysql_init.sh file to create tables.
 
 \c contrib_regression
 CREATE EXTENSION IF NOT EXISTS mysql_fdw;
@@ -37,9 +37,10 @@ CREATE FOREIGN TABLE fdw193_ft1(stu_id varchar(10), stu_name varchar(255), stu_d
 
 -- Operation on blob data.
 INSERT INTO f_empdata VALUES (1, decode ('01234567', 'hex'));
+INSERT INTO f_empdata VALUES (2, 'abc');
 SELECT count(*) FROM f_empdata ORDER BY 1;
 SELECT emp_id, emp_dat FROM f_empdata ORDER BY 1;
-UPDATE f_empdata SET emp_dat = decode ('0123', 'hex');
+UPDATE f_empdata SET emp_dat = decode ('0123', 'hex') WHERE emp_id = 1;
 SELECT emp_id, emp_dat FROM f_empdata ORDER BY 1;
 
 -- FDW-126: Insert/update/delete statement failing in mysql_fdw by picking
@@ -62,8 +63,21 @@ SELECT a, b FROM f_mysql_test ORDER BY 1, 2;
 -- Insert into fdw126_ft2 table which does not have dbname specified while
 -- creating the foreign table, so it will consider the schema name of foreign
 -- table as database name and try to connect/lookup into that database.  Will
--- throw an error.
-INSERT INTO fdw126_ft2 VALUES(2, 'Two');
+-- throw an error. The error message is different on old mysql and mariadb
+-- servers so give the generic message.
+DO
+$$
+BEGIN
+  INSERT INTO fdw126_ft2 VALUES(2, 'Two');
+  EXCEPTION WHEN others THEN
+	IF SQLERRM LIKE '%SELECT command denied to user ''%''@''%'' for table ''student''' THEN
+	  RAISE NOTICE E'failed to execute the MySQL query: \nUnknown database ''public''';
+    ELSE
+	  RAISE NOTICE '%', SQLERRM;
+	END IF;
+END;
+$$
+LANGUAGE plpgsql;
 
 -- Check with the same table name from different database. fdw126_ft3 is
 -- pointing to the mysql_fdw_regress1.numbers and not mysql_fdw_regress.numbers
@@ -74,8 +88,22 @@ UPDATE fdw126_ft3 SET b = 'one' WHERE a = 1;
 DELETE FROM fdw126_ft3 WHERE a = 1;
 
 -- Check when table_name is given in database.table form in foreign table
--- should error out as syntax error
-INSERT INTO fdw126_ft6 VALUES(1, 'One');
+-- should error out as syntax error. The error contains server name like
+-- MySQL or MariaDB, so give the generic message by removing the server name, so
+-- that it should pass on both the servers.
+DO
+$$
+BEGIN
+  INSERT INTO fdw126_ft6 VALUES(1, 'One');
+  EXCEPTION WHEN others THEN
+	IF SQLERRM LIKE '%You have an error in your SQL syntax; check the manual % for the right syntax to use near ''.student'' at line 1' THEN
+	  RAISE NOTICE E'failed to execute the MySQL query: \nYou have an error in your SQL syntax; check the manual that corresponds to your server version for the right syntax to use near ''.student'' at line 1';
+    ELSE
+	  RAISE NOTICE '%', SQLERRM;
+	END IF;
+END;
+$$
+LANGUAGE plpgsql;
 
 -- Perform the ANALYZE on the foreign table which is not present on the remote
 -- side.  Should not crash.
@@ -106,8 +134,6 @@ BEFORE UPDATE ON fdw126_ft1
 FOR EACH ROW EXECUTE PROCEDURE before_row_update_func();
 
 INSERT INTO fdw126_ft1 VALUES(1, 'One', 101);
-EXPLAIN (verbose, costs off)
-UPDATE fdw126_ft1 SET stu_dept = 201 WHERE stu_id = 1;
 UPDATE fdw126_ft1 SET stu_dept = 201 WHERE stu_id = 1;
 SELECT * FROM fdw126_ft1 ORDER BY stu_id;
 
@@ -140,8 +166,6 @@ BEFORE UPDATE ON fdw193_ft1
 FOR EACH ROW EXECUTE PROCEDURE before_row_update_func();
 
 INSERT INTO fdw193_ft1 VALUES('aa', 'One', 101);
-EXPLAIN (verbose, costs off)
-UPDATE fdw193_ft1 SET stu_dept = 201 WHERE stu_id = 'aa';
 UPDATE fdw193_ft1 SET stu_dept = 201 WHERE stu_id = 'aa';
 SELECT * FROM fdw193_ft1 ORDER BY stu_id;
 
@@ -168,6 +192,31 @@ $$ language plpgsql;
 
 UPDATE fdw193_ft1 SET stu_dept = 401 WHERE stu_id = 'aa';
 
+-- FDW-224 Fix COPY FROM and foreign partition routing result in server crash
+-- Should fail as foreign table direct copy not supported
+COPY f_mysql_test TO stdout;
+COPY f_mysql_test (a) TO stdout;
+
+-- Should pass
+COPY (SELECT * FROM f_mysql_test) TO stdout;
+COPY (SELECT a FROM f_mysql_test) TO '/tmp/copy_test.txt' delimiter ',';
+
+-- Should give error message as copy from with foreign table not supported
+DO
+$$
+BEGIN
+  COPY f_mysql_test(a) FROM '/tmp/copy_test.txt' delimiter ',';
+  EXCEPTION WHEN others THEN
+	IF SQLERRM = 'COPY and foreign partition routing not supported in mysql_fdw' OR
+	   SQLERRM = 'cannot copy to foreign table "f_mysql_test"' THEN
+	   RAISE NOTICE 'ERROR:  COPY and foreign partition routing not supported in mysql_fdw';
+        ELSE
+	   RAISE NOTICE '%', SQLERRM;
+	END IF;
+END;
+$$
+LANGUAGE plpgsql;
+
 -- Cleanup
 DELETE FROM fdw126_ft1;
 DELETE FROM f_empdata;
