@@ -51,7 +51,7 @@ ALTER FOREIGN TABLE fdw139_t1 ALTER COLUMN c4 type user_enum;
 ALTER FOREIGN TABLE fdw139_t2 ALTER COLUMN c4 type user_enum;
 
 -- Join two tables
--- target list order is different for v10 and v96.
+-- target list order is different for v10.
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
@@ -62,7 +62,7 @@ SELECT t1.c1, t2.c1
 
 -- INNER JOIN with where condition.  Should execute where condition separately
 -- on remote side.
--- target list order is different for v10 and v96.
+-- target list order is different for v10.
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1) WHERE t1.c2 = 100
@@ -72,7 +72,7 @@ SELECT t1.c1, t2.c1
   ORDER BY t1.c3, t1.c1;
 
 -- INNER JOIN in which join clause is not pushable.
--- target list order is different for v10 and v96.
+-- target list order is different for v10.
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (abs(t1.c1) = t2.c1) WHERE t1.c2 = 100
@@ -82,7 +82,7 @@ SELECT t1.c1, t2.c1
   ORDER BY t1.c3, t1.c1;
 
 -- Join three tables
--- target list order is different for v10 and v96.
+-- target list order is different for v10.
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c2, t3.c3
   FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1) JOIN fdw139_t3 t3 ON (t3.c1 = t1.c1)
@@ -104,10 +104,10 @@ SELECT t1.c1, t2.c1, t3.c1
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 LEFT JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
-  ORDER BY t1.c1, t2.c1 NULLS LAST;
+  ORDER BY t1.c1, t2.c1 NULLS LAST LIMIT 2 OFFSET 2;
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 LEFT JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
-  ORDER BY t1.c1, t2.c1 NULLS LAST;
+  ORDER BY t1.c1, t2.c1 NULLS LAST LIMIT 2 OFFSET 2;
 
 -- LEFT JOIN evaluating as INNER JOIN, having unsafe join clause.
 EXPLAIN (COSTS false, VERBOSE)
@@ -147,7 +147,7 @@ SELECT t1.c1, t1.c2, t2.c1, t2.c2
   WHERE (t2.c1 < 10 OR t2.c1 IS NULL) AND t1.c1 < 10;
 
 -- RIGHT OUTER JOIN
--- target list order is different for v10 and v96.
+-- target list order is different for v10.
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 RIGHT JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
@@ -158,7 +158,7 @@ SELECT t1.c1, t2.c1
 
 -- Combinations of various joins
 -- INNER JOIN + RIGHT JOIN
--- target list order is different for v10 and v96.
+-- target list order is different for v10.
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c2, t3.c3
   FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1) RIGHT JOIN fdw139_t3 t3 ON (t1.c1 = t3.c1)
@@ -179,7 +179,7 @@ SELECT t1.c1, t2.c1
 
 -- Join two tables with FOR UPDATE clause
 -- tests whole-row reference for row marks
--- target list order is different for v10 and v96.
+-- target list order is different for v10.
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
@@ -188,7 +188,7 @@ SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
   ORDER BY t1.c3, t1.c1 FOR UPDATE OF t1;
 
--- target list order is different for v10 and v96.
+-- target list order is different for v10.
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
@@ -198,7 +198,7 @@ SELECT t1.c1, t2.c1
   ORDER BY t1.c3, t1.c1 FOR UPDATE;
 
 -- Join two tables with FOR SHARE clause
--- target list order is different for v10 and v96.
+-- target list order is different for v10.
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
@@ -207,7 +207,7 @@ SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
   ORDER BY t1.c3, t1.c1 FOR SHARE OF t1;
 
--- target list order is different for v10 and v96.
+-- target list order is different for v10.
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
@@ -241,28 +241,28 @@ SELECT t1, t2, t1.c1
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1
   FROM fdw139_t1 t1 WHERE EXISTS (SELECT 1 FROM fdw139_t2 t2 WHERE t1.c1 = t2.c1)
-  ORDER BY t1.c1 LIMIT 10;
+  ORDER BY t1.c1;
 SELECT t1.c1
   FROM fdw139_t1 t1 WHERE EXISTS (SELECT 1 FROM fdw139_t2 t2 WHERE t1.c1 = t2.c1)
-  ORDER BY t1.c1 LIMIT 10;
+  ORDER BY t1.c1;
 
 -- ANTI JOIN, not pushed down
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1
   FROM fdw139_t1 t1 WHERE NOT EXISTS (SELECT 1 FROM fdw139_t2 t2 WHERE t1.c1 = t2.c2)
-  ORDER BY t1.c1 LIMIT 10;
+  ORDER BY t1.c1 LIMIT 2;
 SELECT t1.c1
   FROM fdw139_t1 t1 WHERE NOT EXISTS (SELECT 1 FROM fdw139_t2 t2 WHERE t1.c1 = t2.c2)
-  ORDER BY t1.c1 LIMIT 10;
+  ORDER BY t1.c1 LIMIT 2;
 
 -- CROSS JOIN can be pushed down
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 CROSS JOIN fdw139_t2 t2
-  ORDER BY t1.c1, t2.c1 LIMIT 10;
+  ORDER BY t1.c1, t2.c1 LIMIT round(5.4) OFFSET 2;
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 CROSS JOIN fdw139_t2 t2
-  ORDER BY t1.c1, t2.c1 LIMIT 10;
+  ORDER BY t1.c1, t2.c1 LIMIT round(5.4) OFFSET 2;
 
 -- CROSS JOIN combined with local table.
 CREATE TABLE local_t1(c1 int);
@@ -271,10 +271,10 @@ INSERT INTO local_t1 VALUES (1), (2);
 EXPLAIN (COSTS false, VERBOSE)
 SELECT t1.c1, t2.c1, l1.c1
   FROM fdw139_t1 t1 CROSS JOIN fdw139_t2 t2 CROSS JOIN local_t1 l1
-  ORDER BY t1.c1, t2.c1, l1.c1 LIMIT 10;
+  ORDER BY t1.c1, t2.c1, l1.c1 LIMIT 8 OFFSET round(2.2);
 SELECT t1.c1, t2.c1, l1.c1
   FROM fdw139_t1 t1 CROSS JOIN fdw139_t2 t2 CROSS JOIN local_t1 l1
-  ORDER BY t1.c1, t2.c1, l1.c1 LIMIT 10;
+  ORDER BY t1.c1, t2.c1, l1.c1 LIMIT 8 OFFSET round(2.2);
 SELECT count(t1.c1)
   FROM fdw139_t1 t1 CROSS JOIN fdw139_t2 t2 CROSS JOIN local_t1 l1;
 
@@ -309,7 +309,7 @@ SELECT t1.c1, t2.c1
 -- in the SELECT clause.  In this test unsafe clause needs to have column
 -- references from both joining sides so that the clause is not pushed down
 -- into one of the joining sides.
--- target list order is different for v10 and v96.
+-- target list order is different for v10.
 EXPLAIN (VERBOSE, COSTS OFF)
 SELECT t1.c1, t2.c1
   FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1) WHERE t1.c4 = t2.c4
@@ -332,36 +332,36 @@ ALTER VIEW v2 OWNER TO regress_view_owner;
 EXPLAIN (VERBOSE, COSTS OFF)
 SELECT t1.c1, t2.c2
   FROM v1 t1 LEFT JOIN v2 t2 ON (t1.c1 = t2.c1)
-  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST LIMIT 10;  -- not pushed down, different view owners
+  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST;  -- not pushed down, different view owners
 SELECT t1.c1, t2.c2
   FROM v1 t1 LEFT JOIN v2 t2 ON (t1.c1 = t2.c1)
-  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST LIMIT 10;
+  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST;
 
 ALTER VIEW v1 OWNER TO regress_view_owner;
 EXPLAIN (VERBOSE, COSTS OFF)
 SELECT t1.c1, t2.c2
   FROM v1 t1 LEFT JOIN v2 t2 ON (t1.c1 = t2.c1)
-  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST LIMIT 10;  -- pushed down
+  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST;  -- pushed down
 SELECT t1.c1, t2.c2
   FROM v1 t1 LEFT JOIN v2 t2 ON (t1.c1 = t2.c1)
-  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST LIMIT 10;
+  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST;
 
 EXPLAIN (VERBOSE, COSTS OFF)
 SELECT t1.c1, t2.c2
   FROM v1 t1 LEFT JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
-  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST LIMIT 10;  -- not pushed down, view owner not current user
+  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST;  -- not pushed down, view owner not current user
 SELECT t1.c1, t2.c2
   FROM v1 t1 LEFT JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
-  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST LIMIT 10;
+  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST;
 
 ALTER VIEW v1 OWNER TO CURRENT_USER;
 EXPLAIN (VERBOSE, COSTS OFF)
 SELECT t1.c1, t2.c2
   FROM v1 t1 LEFT JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
-  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST LIMIT 10;  -- pushed down
+  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST;  -- pushed down
 SELECT t1.c1, t2.c2
   FROM v1 t1 LEFT JOIN fdw139_t2 t2 ON (t1.c1 = t2.c1)
-  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST LIMIT 10;
+  ORDER BY t1.c1, t2.c1, t2.c2 NULLS LAST;
 ALTER VIEW v1 OWNER TO regress_view_owner;
 
 -- Non-Var items in targetlist of the nullable rel of a join preventing
@@ -392,6 +392,43 @@ SELECT fdw139_t3.c1, q.*
   ) q(a, b, c) ON (fdw139_t3.c1 = q.b)
   WHERE fdw139_t3.c1 BETWEEN 10 AND 15;
 
+-- FDW-129: Limit and offset pushdown with join pushdown.
+EXPLAIN (COSTS false, VERBOSE)
+SELECT t1.c1, t2.c1
+  FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (TRUE)
+  ORDER BY t1.c1, t2.c1 LIMIT round(2.2) OFFSET 2;
+SELECT t1.c1, t2.c1
+  FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (TRUE)
+  ORDER BY t1.c1, t2.c1 LIMIT round(2.2) OFFSET 2;
+
+-- Limit as NULL, no LIMIT/OFFSET pushdown.
+EXPLAIN (COSTS false, VERBOSE)
+SELECT t1.c1, t2.c1
+  FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (TRUE)
+  ORDER BY t1.c1, t2.c1 LIMIT NULL OFFSET 1;
+SELECT t1.c1, t2.c1
+  FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (TRUE)
+  ORDER BY t1.c1, t2.c1 LIMIT NULL OFFSET 1;
+
+-- Limit as ALL, no LIMIT/OFFSET pushdown.
+EXPLAIN (COSTS false, VERBOSE)
+SELECT t1.c1, t2.c1
+  FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (TRUE)
+  ORDER BY t1.c1, t2.c1 LIMIT ALL OFFSET 1;
+SELECT t1.c1, t2.c1
+  FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (TRUE)
+  ORDER BY t1.c1, t2.c1 LIMIT ALL OFFSET 1;
+
+
+-- Offset as NULL, no LIMIT/OFFSET pushdown.
+EXPLAIN (COSTS false, VERBOSE)
+SELECT t1.c1, t2.c1
+  FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (TRUE)
+  ORDER BY t1.c1, t2.c1 LIMIT 3 OFFSET NULL;
+SELECT t1.c1, t2.c1
+  FROM fdw139_t1 t1 JOIN fdw139_t2 t2 ON (TRUE)
+  ORDER BY t1.c1, t2.c1 LIMIT 3 OFFSET NULL;
+
 -- Delete existing data and load new data for partition-wise join test cases.
 DROP OWNED BY regress_view_owner;
 DROP ROLE regress_view_owner;
@@ -423,33 +460,14 @@ INSERT INTO tmp_t4 values(8, 8, 'CCC13');
 -- Test partition-wise join
 SET enable_partitionwise_join TO on;
 
--- Create the partition table in plpgsql block as those are failing with
--- different error messages on back-branches.
--- All test cases related to partition-wise join gives an error on v96 and v95
--- as partition syntax is not supported there.
-DO
-$$
-BEGIN
-  EXECUTE 'CREATE TABLE fprt1 (c1 int, c2 int, c3 varchar, c4 varchar) PARTITION BY RANGE(c1)';
-EXCEPTION WHEN others THEN
-  RAISE NOTICE 'syntax error';
-END;
-$$
-LANGUAGE plpgsql;
+-- Create the partition table.
+CREATE TABLE fprt1 (c1 int, c2 int, c3 varchar, c4 varchar) PARTITION BY RANGE(c1);
 CREATE FOREIGN TABLE ftprt1_p1 PARTITION OF fprt1 FOR VALUES FROM (1) TO (4)
   SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'test1');
 CREATE FOREIGN TABLE ftprt1_p2 PARTITION OF fprt1 FOR VALUES FROM (5) TO (8)
   SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', TABLE_NAME 'test2');
 
-DO
-$$
-BEGIN
-  EXECUTE 'CREATE TABLE fprt2 (c1 int, c2 int, c3 varchar) PARTITION BY RANGE(c2)';
-EXCEPTION WHEN syntax_error THEN
-  RAISE NOTICE 'syntax error';
-END;
-$$
-LANGUAGE plpgsql;
+CREATE TABLE fprt2 (c1 int, c2 int, c3 varchar) PARTITION BY RANGE(c2);
 CREATE FOREIGN TABLE ftprt2_p1 PARTITION OF fprt2 FOR VALUES FROM (1) TO (4)
   SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'test3');
 CREATE FOREIGN TABLE ftprt2_p2 PARTITION OF fprt2 FOR VALUES FROM (5) TO (8)
