@@ -318,6 +318,18 @@ SELECT * FROM enum_t2 ORDER BY id;
 DROP FOREIGN TABLE enum_t1;
 DROP FOREIGN TABLE enum_t2;
 
+-- FDW-217: IMPORT FOREIGN SCHEMA command with 'import_enum_as_text' option as
+-- true should map MySQL ENUM type to TEXT type in PG.
+IMPORT FOREIGN SCHEMA mysql_fdw_regress LIMIT TO (enum_t1, enum_t2)
+  FROM SERVER mysql_svr INTO public OPTIONS (import_enum_as_text 'true');
+SELECT attrelid::regclass, atttypid::regtype FROM pg_attribute
+  WHERE (attrelid = 'enum_t1'::regclass OR attrelid = 'enum_t2'::regclass) AND
+    attnum > 1 ORDER BY 1;
+SELECT * FROM enum_t1 ORDER BY id;
+SELECT * FROM enum_t2 ORDER BY id;
+DROP FOREIGN TABLE enum_t1;
+DROP FOREIGN TABLE enum_t2;
+
 -- FDW-248: IMPORT FOREIGN SCHEMA command should work correctly if called
 -- multiple times. Earlier we wrongly used PG_TRY/CATCH block to get the server
 -- options without clearing the error state and that exceeded
@@ -427,6 +439,82 @@ IMPORT FOREIGN SCHEMA mysql_fdw_regress LIMIT TO (test_set, mysql_test, test_tbl
 SELECT relname FROM pg_class
   WHERE relname IN ('test_set', 'mysql_test', 'test_tbl1') AND relnamespace = 'public'::regnamespace;
 
+-- FDW-417: Updating a NOT NULL column with NULL should throw an error
+-- if we set sql_mode to STRICT_ALL_TABLES.
+ALTER SERVER mysql_svr OPTIONS (sql_mode 'ANSI_QUOTES,STRICT_ALL_TABLES');
+UPDATE f_mysql_test SET b = NULL WHERE a = 1;
+SELECT * FROM f_mysql_test ORDER BY 1;
+ALTER SERVER mysql_svr OPTIONS (DROP sql_mode);
+UPDATE f_mysql_test SET b = NULL WHERE a = 1;
+SELECT * FROM f_mysql_test ORDER BY 1;
+UPDATE f_mysql_test SET b = 1 WHERE a = 1;
+-- We should get a run-time error when sql_mode is set to invalid value.
+ALTER SERVER mysql_svr OPTIONS (sql_mode 'ABCD');
+SELECT * FROM f_mysql_test ORDER BY 1;
+ALTER SERVER mysql_svr OPTIONS (DROP sql_mode);
+
+-- FDW-426: The numeric value should display correctly per precision and scale
+-- defined.
+CREATE FOREIGN TABLE f_test6(c1 numeric(6,2))
+  SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'test6');
+SELECT * FROM f_test6 ORDER BY 1;
+
+-- Number with the required precision.
+DROP FOREIGN TABLE f_test6;
+CREATE FOREIGN TABLE f_test6(c1 numeric(6,4))
+  SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'test6');
+SELECT * FROM f_test6 ORDER BY 1;
+
+-- Number only with precision.
+DROP FOREIGN TABLE f_test6;
+CREATE FOREIGN TABLE f_test6(c1 numeric(6))
+  SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'test6');
+SELECT * FROM f_test6 ORDER BY 1;
+
+-- Number with improper precision and scale, should throw an error.
+DROP FOREIGN TABLE f_test6;
+CREATE FOREIGN TABLE f_test6(c1 numeric(3,2))
+  SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'test6');
+SELECT * FROM f_test6 ORDER BY 1;
+
+-- FDW-156: Long string data in column greater than MAXDATALEN length should
+-- be fetched correctly.
+CREATE FOREIGN TABLE f_test7(c1 int, c2 text)
+  SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'test7');
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT c1, length(c2) FROM f_test7 ORDER BY 1;
+SELECT c1, length(c2) FROM f_test7 ORDER BY 1;
+
+-- FDW-130: ORDER BY with collation clause, should not get pushdown.
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT * FROM f_test_tbl1 ORDER BY c2 COLLATE "en_US";
+SELECT * FROM f_test_tbl1 ORDER BY c2 COLLATE "en_US";
+
+-- Order by desc
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT * FROM f_test_tbl1 ORDER BY c1 DESC;
+SELECT * FROM f_test_tbl1 ORDER BY c1 DESC;
+
+-- Order by is not null
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT * FROM f_test_tbl1 ORDER BY c7 IS NOT NULL;
+SELECT * FROM f_test_tbl1 ORDER BY c7 IS NOT NULL;
+
+-- Order by is not null desc
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT * FROM f_test_tbl1 ORDER BY c7 IS NOT NULL DESC;
+SELECT * FROM f_test_tbl1 ORDER BY c7 IS NOT NULL DESC;
+
+-- Order by desc nulls first
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT * FROM f_test_tbl1 ORDER BY c7 DESC NULLS FIRST;
+SELECT * FROM f_test_tbl1 ORDER BY c7 DESC NULLS FIRST;
+
+-- Order by asc nulls last
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT * FROM f_test_tbl1 ORDER BY c7 DESC NULLS LAST;
+SELECT * FROM f_test_tbl1 ORDER BY c7 DESC NULLS LAST;
+
 -- Cleanup
 DROP TABLE l_test_tbl1;
 DROP TABLE l_test_tbl2;
@@ -450,6 +538,8 @@ DROP FOREIGN TABLE test5_1;
 DROP FOREIGN TABLE test5_2;
 DROP FOREIGN TABLE mysql_test;
 DROP FOREIGN TABLE test_tbl1;
+DROP FOREIGN TABLE f_test6;
+DROP FOREIGN TABLE f_test7;
 DROP TYPE size_t;
 DROP TYPE enum_t1_size_t;
 DROP TYPE enum_t2_size_t;
